/**
 * Copyright (C) 2022 Xilinx, Inc
 * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

#pragma once

// Include files
// Please keep these to the bare minimum
#include "MA35Device.h"

#include <string>
#include <memory>
#include <map>
#include <iostream>
#include <vector>
#include <boost/property_tree/ptree.hpp>
#include <boost/program_options.hpp>
#include <boost/algorithm/string.hpp>

#if __has_include("jsf_mautil.h")
#include "jsf_mautil.h"
namespace ma35 = jsf_mautil;
#else
#include "jsf_mamgmt.h"
namespace ma35 = jsf_mamgmt;
#endif

namespace XBUtilities {

void can_proceed_or_throw(const std::string& info, const std::string& error);

void sudo_or_throw(const std::string& msg);
void print_exception_and_throw_cancel(const std::runtime_error& e);

void check_bdf(const std::string& bdf, bool inUserDomain);

void collect_devices(const std::set<std::string>& deviceBDFs, bool inUserDomain, DeviceCollection& deviceCollection);

boost::property_tree::ptree get_available_devices(bool inUserDomain);
bool                        dev_present(std::string bdf, bool userDomain);
void                        list_devices(std::ostream& out, bool user);

void pt_to_map(const boost::property_tree::ptree& pt_in, uint32_t depth, const std::string& path, std::unordered_map<std::string, uint32_t>& map_out);

void get_os_info(boost::property_tree::ptree& pt);

void get_host_info(boost::property_tree::ptree& pt);

void get_version_info(boost::property_tree::ptree& pt);

void get_mem_info(const std::shared_ptr<MA35Device> dev, boost::property_tree::ptree& pt);

void get_pcie_info(const std::shared_ptr<MA35Device> dev, boost::property_tree::ptree& pt);

void get_thermal_info(const std::shared_ptr<MA35Device> dev, boost::property_tree::ptree& pt);

void get_power_info(const std::shared_ptr<MA35Device> dev, boost::property_tree::ptree& pt);

void get_device_info(const std::shared_ptr<MA35Device> dev, boost::property_tree::ptree& pt);

void get_error_info(const std::shared_ptr<MA35Device> dev, boost::property_tree::ptree& pt);

void do_reset(const std::shared_ptr<MA35Device> dev, std::string& type, boost::property_tree::ptree& pt);

void do_numvfs(const std::shared_ptr<MA35Device> dev, int32_t value, boost::property_tree::ptree& pt);

//xrt_core::query::reset_type str_to_reset_obj(const std::string& str);

std::string get_version();

/**
   * OEM ID is a unique number called as the 
   * Private Enterprise Number (PEN) maintained by IANA
   * 
   * Return: Manufacturer's name
   */
std::string parse_oem_id(const std::string& oemid);

enum class unit { bytes, Hertz };
uint64_t string_to_base_units(std::string str, const unit& conversion_unit);

inline bool is_power_of_2(const uint64_t x) {
  /*
    * Verify that the given value is greater than zero
    * and that only one bit is set in the given value
    * if only one bit is set this implies the given value is a power of 2
    */
  return (x != 0) && ((x & (x - 1)) == 0);
}

}; // namespace XBUtilities
