/**
 * Copyright (C) 2020-2022 Xilinx, Inc
 * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

// ------ I N C L U D E   F I L E S -------------------------------------------
// Local - Include Files
#include "ReportHost.h"
#include "MAUtilitiesCore.h"
#include "MAUtilities.h"

// 3rd Party Library - Include Files
#include <string>
#include <boost/format.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/property_tree/json_parser.hpp>

#define BYTES_TO_MEGABYTES 0x100000ll

void ReportHost::getPropertyTree(const std::shared_ptr<MA35Device> dev __attribute__((unused)), boost::property_tree::ptree& _pt) const {
  boost::property_tree::ptree pt;
  boost::property_tree::ptree empty_pt;
  boost::property_tree::ptree pt_os_info;

  XBUtilities::get_host_info(pt);

  // There can only be 1 root node
  _pt.add_child("host", pt);
}

void ReportHost::writeReport(const boost::property_tree::ptree& _pt, std::ostream& _output) const {
  boost::property_tree::ptree empty_ptree;

  boost::property_tree::ptree pt;
  XBUtilities::get_version_info(pt);

  //For debug TODO
  //boost::property_tree::write_json(_output, _pt, true /*Pretty print*/);
  //_output << std::endl;
  _output << "System Configuration\n";
  if (_pt.get<std::string>("host.status") != "ok") {
    _output << "ERROR " << _pt.get<std::string>("host.status") << std::endl;
    return;
  }

  try {
    _output << boost::format("  %-20s : %s\n") % "OS Name" % _pt.get<std::string>("host.os.os", "??");
    _output << boost::format("  %-20s : %s\n") % "Release" % _pt.get<std::string>("host.os.release", "??");
    _output << boost::format("  %-20s : %s\n") % "Version" % _pt.get<std::string>("host.os.version", "??");
    _output << boost::format("  %-20s : %s\n") % "Machine" % _pt.get<std::string>("host.os.machine", "??");
    _output << boost::format("  %-20s : %s\n") % "CPU Cores" % _pt.get<std::string>("host.os.cpu_cores", "??");
    _output << boost::format("  %-20s : %lld MB\n") % "Memory" %
            (std::strtoll(_pt.get<std::string>("host.os.memory_bytes", "0").c_str(), nullptr, 10) / BYTES_TO_MEGABYTES);
    _output << boost::format("  %-20s : %s\n") % "glibc" % _pt.get<std::string>("host.os.glibc", "N/A");
    _output << boost::format("  %-20s : %s\n") % "MA35D Version" % pt.get<std::string>("ver.pkg_version", "N/A");

    _output << std::endl;
#if 0
    _output << "MA35\n";
    _output << boost::format("  %-20s : %s\n") % "Version" % _pt.get<std::string>("host.MA35.version", "N/A");
    _output << boost::format("  %-20s : %s\n") % "Branch" % _pt.get<std::string>("host.MA35.branch", "N/A");
    _output << boost::format("  %-20s : %s\n") % "Hash" % _pt.get<std::string>("host.MA35.hash", "N/A");
    _output << boost::format("  %-20s : %s\n") % "Hash Date" % _pt.get<std::string>("host.MA35.build_date", "N/A");
#endif
    const boost::property_tree::ptree& available_drivers = _pt.get_child("host.MA35.drivers", empty_ptree);
    for (auto& drv : available_drivers) {
      _output << boost::format("  %-20s : %s, %s\n") % drv.first % drv.second.get_value<std::string>() % "N/A";
    }
    _output << std::endl;
  } catch (const boost::property_tree::ptree_error& ex) {
    throw std::runtime_error(boost::str(boost::format("%s. Please contact your AMD representative to fix the issue") % ex.what()));
  }

  _output << "Devices present\n";
  const boost::property_tree::ptree& available_devices = _pt.get_child("host.devices", empty_ptree);

  if (available_devices.empty())
    _output << "  0 devices found" << std::endl;

  for (const auto& kd : available_devices) {
    const boost::property_tree::ptree& dev = kd.second;
    _output << "  device bdf: "
            << "[" << dev.get<std::string>("bdf") << "]" << std::endl;
  }
}
