// Copyright (C) 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#pragma once

extern "C" {
#include <pci/pci.h>
}

#include <vector>

#include <jsf_int.h>

namespace jsf_helpers {

static constexpr uint32_t MAX_TRANS_DEVICES = 16;
std::vector<pci_dev>      GetPciDevices(int Vendor, int Device);
std::string               DbdfString(const pci_dev& Device);

class NodeChildCount : public jsf_int::NodeLeaf {
private:
  const jsf_int::NodeInternal* m_node;

public:
  NodeChildCount(const std::string& Name, const jsf_int::NodeInternal* Node) : NodeLeaf(Name), m_node(Node) {
  }
  virtual json Invoke(const json& /* Parameters */ = {}, bool /* Iteration */ = false) override final { return m_node->m_children.size(); }
};

class NodeRawFile : public jsf_int::NodeLeaf {
private:
  std::string m_path;

public:
  NodeRawFile(const std::string& Name, const std::string& Path) : NodeLeaf(Name), m_path(Path) {
  }

  virtual json Invoke(const json& /* Parameters */ = {}, bool /* Iteration */ = false) override { // TODO(jkuhn): error on parameters present
    std::ifstream ifs(m_path.c_str(), std::ios::in | std::ios::ate);                              // could use mmap, this is good enough
    if (!ifs.is_open()) {
      throw std::runtime_error("Contents missing");
    }
    auto size = ifs.tellg();
    ifs.seekg(0, std::ios::beg);
    std::string contents;
    contents.resize(size);
    ifs.read(contents.data(), size);
    return contents;
  }
};

class NodeJsonFile : public NodeRawFile {
public:
  NodeJsonFile(const std::string& Name, const std::string& Path) : NodeRawFile(Name, Path) {
  }

  virtual json Invoke(const json& Parameters = {}, bool Iteration = false) override final { // TODO(jkuhn): error on parameters present
    return json::parse(std::string(NodeRawFile::Invoke(Parameters, Iteration)));
  }
};

class NodeVersionInfo : public jsf_helpers::NodeRawFile {
public:
  explicit NodeVersionInfo(const std::string& Name, const std::string& VersionFile);
  virtual json Invoke(const json& Parameters, bool Iteration) override;
};

class NodeSystemInfo : public jsf_int::NodeJson {
public:
  explicit NodeSystemInfo(const std::string& Name);
};

class NodePcieInfo : public jsf_int::NodeJson {
public:
  explicit NodePcieInfo(const std::string& Name, const pci_dev& Device);
};

class NodeTelemetry : public jsf_int::NodeLeaf {
private:
  std::string m_location;
  std::string m_valueString;
  bool        m_demo;

protected:
  NodeTelemetry(const std::string& Name, const std::string& Location, const std::string& ValueString, bool Demo) :
      NodeLeaf(Name), m_location(Location), m_valueString(ValueString), m_demo(Demo) {
      }

  virtual json Invoke(const json& /* Parameters */, bool /* Iteration */) override {
    json ret;
    ret["id"]          = m_location;
    ret[m_valueString] = GetValue(m_demo);
    return ret;
  }

  virtual json GetValue(bool Demo) = 0;
};

class NodeTemperatureBase : public NodeTelemetry {
protected:
  NodeTemperatureBase(const std::string& Name, const std::string& Location, bool Demo) : NodeTelemetry(Name, Location, "temp_C", Demo) {
  }
};

class NodeVoltageBase : public NodeTelemetry {
protected:
  NodeVoltageBase(const std::string& Name, const std::string& Location, bool Demo) : NodeTelemetry(Name, Location, "volt_mV", Demo) {
  }
};

} // namespace jsf_helpers
