// Copyright (C) 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#include <fcntl.h>
#include <sys/stat.h>

#include <Logging.h>

#include "Device.h"

Device::Device(const pci_dev& PciDevice, bool Demo) : m_dbdf(jsf_helpers::DbdfString(PciDevice)), m_pciDevice(PciDevice), m_demo(Demo) {
}

#if 0
uint32_t Device::GetDeviceVoltage(unsigned Index) const {
  if (m_demo) {
    return 1600 + (rand() & 31) - 16;
  }

  auto volt = ReadBar2(S1_SYS_CON_OFF + MAILBOX1_OFFSET + 0x110 + (Index * 4)); // Chip Voltage starts at 272 (0x110)
  return (((volt * 315) * ((Device::PVT_VREF * 1000) / 1290)) / 1000000);
}

uint32_t Device::GetDeviceTemperature(unsigned Index) const {
  if (m_demo) {
    return 60 + (rand() & 31) - 16;
  }

  auto tempr = ReadBar2(S1_SYS_CON_OFF + MAILBOX1_OFFSET + 0x120 + (Index * 4)); // Chip Temperature starts at 288 (0x120)
  return (((tempr - Device::PVT_DOTP25) * 625) / 10000 + 25);
}
#endif

uint32_t Device::GetSensorInfo(unsigned Index, unsigned Offset) const {
  if (m_demo) {
    return 60 + (rand() & 71) - 16;
  }
  auto btempr = ReadBar4(Offset + (Index * 4));
  return btempr;
}

uint32_t Device::GetBoardVcInfo(unsigned Index, unsigned Offset) const {
  if (m_demo) {
    return 60 + (rand() & 71) - 16;
  }

  auto board_vc = 0;
  if (Index % 2) {
    board_vc = ReadBar4(Offset + ((Index - Index / 2 - 1) * 4));
    board_vc = (board_vc >> 16) & 0xFFFF;
  } else {
    board_vc = ReadBar4(Offset + ((Index / 2) * 4));
    board_vc = board_vc & 0xFFFF;
  }
  return board_vc;
}

uint32_t Device::GetUptime(unsigned Offset) const {
  if (m_demo) {
    return rand();
  }

  auto upt = ReadBar4(Offset);
  return upt;
}

std::string Device::GetVersion(unsigned Index, unsigned Offset) const {
  if (m_demo) {
    return std::to_string(rand() & 9) + "." + std::to_string(rand() & 9) + "." + std::to_string(rand() & 9);
  }

  char version[32];
  auto version_info = ReadBar4(Offset + (Index * 4));
  snprintf(version, sizeof(version), "%u.%u.%u", (version_info >> 16) & 0xFF, (version_info >> 8) & 0xFF, (version_info & 0xFF));
  return version;
}

std::string Device::GetProductInfo(unsigned Index, unsigned Offset) const {
  if (m_demo) {
    return std::to_string(400 + (rand() & 255));
  }

  if (Index == 1) {
    std::stringstream stream;
    stream << std::hex << ReadBar4(Offset);
    return ("0x" + std::string(stream.str()));
  }

  char product_info[32] = {};
  for (unsigned int i = 0; i < Index; i++) {
    auto info = ReadBar4(Offset + (i * 4));
#if 0
    logDebug("info: %d %d %d %d", tmp_info1, tmp_info2, tmp_info3, tmp_info4);
#endif
    product_info[i * 4 + 0] = info & 0xFF;
    product_info[i * 4 + 1] = (info >> 8) & 0xFF;
    product_info[i * 4 + 2] = (info >> 16) & 0xFF;
    product_info[i * 4 + 3] = (info >> 24) & 0xFF;
  }
  return product_info;
}

std::string Device::GetPcieInfo(const std::string& Val) const {
  if (m_demo) {
    return std::to_string(400 + (rand() & 255));
  }

  auto val_rawFile = Val;
  if (!val_rawFile.empty()) {
    auto pos = val_rawFile.find_first_of('.', 0);
    if (pos != std::string::npos) {
      val_rawFile = val_rawFile.substr(0, pos) + "GT/s";
    } else {
      val_rawFile = val_rawFile.substr(0, val_rawFile.find_first_of('\n', 0) - 0);
    }
  }
  // return (val_rawFile.empty()) ? "ERROR" : val_rawFile;
  return val_rawFile;
}

unsigned long Device::GetDdrInfo(unsigned Index, unsigned Offset) const {
  if (m_demo) {
    return (400 + (rand() & 1023));
  }

  unsigned long val           = 0;
  int           cnt           = 0;
  int           slice2_offset = 0;
  if (Index > 3) {
    slice2_offset = 80;
    Index %= 4;
  }
  for (int ddr_cnt_slice = 0; ddr_cnt_slice < 4; ddr_cnt_slice++, cnt += 20) {
#if 0
    logNotice("ddr_cnt_slice=%d ; slice2_offset=%d ; cnt = %d ; Index=%d", ddr_cnt_slice, slice2_offset, cnt, Index);
#endif

    unsigned long timer = ReadBar4(Offset + slice2_offset + cnt);
    unsigned long count = ReadBar4(Offset + slice2_offset + ((Index + 1) * 4) + cnt);

    if (timer == 0) {
      throw std::runtime_error("DDR Bandwidth: Timer Reading Failed (Check Driver/Firmware)!");
    }
    val += (count * Device::DDR_CTRL_FREQ) / timer;
  }
  return val;
}

std::pair<unsigned int, unsigned int> Device::GetErrorCounters(unsigned Index, unsigned Offset) const {
  if (m_demo) {
    return std::make_pair((rand() & 255), (rand() & 255));
  }

  auto error_counter_info = ReadBar4(Offset + (Index * 4));
  return std::make_pair((error_counter_info >> 16) & 0xFFFF, error_counter_info & 0xFFFF);
}
